unit Parser;

{$mode objfpc}{$H+}

interface



uses
  Classes, SysUtils;

// modified to allow variables such as x1, y0 to be all alpha and not split to next token


const
  MAX_TOKENS = 127;
  alphaset = ['A'..'Z', 'a'..'z','_','0'..'9', '.'];
  numset = ['0'..'9', '.'];
  opset = ['+','-','%','/','|',' ','=','>','<'];
  delimset = [#9,','];
  //commentset = ['//'];

type

  //tokentype = record
  //  T:      array[0..MAX_TOKENS] of string [128];
  //end;

  tokentype = array[0..MAX_TOKENS] of string [128];

  function parse(buffer: string; var token: tokentype ) : boolean;



implementation


// split text line into alpha, numeric, and operators
// each entity gets its own token.  Tokens are changed only when type of alpha or numeric is changed
// delimiters are ignored.

function parse(buffer: string; var token: tokentype ) : boolean;
var
   i:                           integer;
   which:                       integer;
   intext:                      boolean;
   alpha:                       boolean;
   old_alpha:                   boolean;
   scan_char:                   char;
   endmarker:                   integer;
   j:                           integer;

begin
  for i := 0 to MAX_TOKENS-1 do token[i] := '';

  if length(buffer) = 0 then
  begin
    result := false;
  end
  else
  begin

    old_alpha := false;
    i := 1;
    which := 0;

    // set alpha according to buffer
    //if buffer[i] in alphaset then alpha := true;
    //if buffer[i] in numset then alpha := false;

    // be consistent about it
    //old_alpha := alpha;
    while (i <= length(buffer)) and (which < MAX_TOKENS) do
    begin
      scan_char := buffer[i];
      if scan_char in alphaset then
      begin
        // buffer[i] is alpha
        if old_alpha = false then
        begin
          // was doing numbers, skip over to letters
          inc(which);
          token[which] := token[which] + scan_char;
          old_alpha := true;
          inc(i);
        end
        else
        begin
          // still doing letters
          token[which] := token[which] + scan_char;
          inc(i);
        end;
      end
      else
      begin
        // buffer[i] is not alpha
        if scan_char in numset then
        begin
          // buffer[i] is a number
          if old_alpha = true then
          begin
            // was doing letters, skip over to numbers
            inc(which);
            token[which] := token[which] + scan_char;
            old_alpha := false;
            inc(i);
          end
          else
          begin
            // still doing numbers
            token[which] := token[which] + scan_char;
            inc(i);
          end;
        end
        else
        begin
          // buffer[i] is either unknown, operator, or delimiter
          if scan_char in opset then
          begin
            // buffer[i] is an operator
            inc(which);
            token[which] := token[which] + scan_char;
            inc (which);
            inc(i);
          end
          else
          begin
            if scan_char in delimset then
            begin
              // buffer[i] is a delimiter go to next token but don't store it
              inc(which);
              inc(i);
            end
            else
            begin
              // buffer[i] is not categorized, just skip it
              inc(i);
            end;
          end
        end;
      end;
    end;            // while
    // strip out nulls and single spaces left by parser

    // find end of tokens
    endmarker := 127;
    while (token[endmarker] = '') do dec(endmarker);

    // starting at beginning
    i := 0;
    // check up to end marker (not null)
    while (i  < endmarker) do
    begin
      if (token[i] = ' ') or (token[i] = '') then
      begin
        // strip out by bubbling down by one
        for j := i to (endmarker-1) do
        begin
          token[j] := token[j+1];
        end;
        // reset duplicate of endmarker token and bring endmarker in
        // to avoid testing trailing nulls
        token[endmarker] := '';
        dec(endmarker);
      end
      else
      begin
        inc(i);
      end;
    end;


  end;
end;


end.

